﻿define(["sitecore", "knockout", "jquery", "underscore", "/sitecore/shell/client/Stylelabs/Base/Api.js"], function (Sitecore, ko, $, _, Api) {
    if (localStorage) {
        localStorage.setItem("pageSize", 10);
        localStorage.setItem("currentPage", 0);
    } else {
        throw "Browser not supported."
    }

    var imageMimeTypes = ["image/jpeg", "image/png"];

    window.ko = ko;
    var SearchResults = Sitecore.Definitions.App.extend({
        baseUrl: null,
        searchPage: null,
        searchComponentId: null,
        userProfile: null,
        savedSearch: null,
        savedSelection: null,
        searchTerm: null,
        currentSavedSearchPage: 0,
        currentSavedSelectionPage: 0,
        selectedAssetId: null,
        searchUrl: null,
        initialize: function () {
        },
        initialized: function () {
            var self = this;

            // Setup the localized tooltips. Is there a better way to do that ?
            $("div[data-sc-id='SearchBox'] input").attr("placeholder", self.SearchMText.get("text"));
            self.SearchBox.set("Tooltip", self.SearchText.get("text"));
            self.OkButton.set("Tooltip", self.AddAssetText.get("text"));

            //UI
            self.getAuthenticationDetails().done(function (success) {
                if (!success) {
                    return;
                }

                self.getUserProfile().done(function () {
                    self.getSearchComponentId().done(function () {
                        self.SearchResults.on('change:selectedItem', self.selectedItemIdChange, self);
                        self.Collections.on('change:selectedItem', self.setSavedSelection, self);
                        self.SavedSearches.on('change:selectedItem', self.setSavedSearch, self);
                        self.PublicLinks.on('change:selectedItem', self.setPublicLink, self);
                        self.getCollections();
                        self.getSavedSearches();
                        self.getAssets();
                    });
                });
            });
        },
        getFileType: function (url) {
            // We are caching GET, but we shouldn't cache HEAD because chrome has a bug where it won't store the CORS header
            // As a GET request will override the cache control of the HEAD request, we need to fool the browser by making him thinking we are doing 2 requests on 2 different resource
            var url = url + "head=true";
            var ajaxRequest = Api.head(url).then(function (message, text, jqXHR) {
                var contentType = jqXHR.getResponseHeader("content-type");

                var itemFound = imageMimeTypes.find(function (mimeType) {
                    return contentType.indexOf(mimeType) !== -1;
                });

                if (itemFound) {
                    return "Image";
                }
                else {
                    return "Other";
                }
            });

            return ajaxRequest
        },

        showWorkspace: function () {
            var self = this;
            window.open(self.searchUrl);
        },
        selectedItemIdChange: function (component, obj) {
            var self = this;
            // only change selected item when triggered by clicking some other item
            // this gets triggered with empty object when clicking twice on the thumbnail
            // (clicking on hyperlink under image is also triggerent same behavior)
            if (obj !== "") {
                self.selectedAssetId = obj.id;
                if (self.selectedAssetId) {
                    if (this.OkButton.viewModel.isEnabled() === true) {
                        localStorage.removeItem("selectedpubliclink");
                        this.OkButton.viewModel.isEnabled(false);
                    }
                    this.getPublicLinksForAsset(self.selectedAssetId);
                }
            }
        },
        returnResult: function (result) {
            var radWindow;
            if (window.radWindow) {
                radWindow = window.radWindow;
                radWindow.Close(result);
            }
            else if (window.frameElement && window.frameElement.radWindow) {
                radWindow = window.frameElement.radWindow;
                radWindow.Close(result);
            }
            else {
                result = JSON.stringify(result);
                window.returnValue = result;
                window.top.returnValue = result;
                window.top.dialogClose();

            }
        },
        okClicked: function () {
            var self = this;
            var returnVal = localStorage.getItem("selectedpubliclink");
            if (!returnVal || !returnVal.length) throw "Public link not found. Please select one.";
            self.clearSession();
            var downloadText = self.DownloadText.get("text");
            self.getFileType(returnVal).done(function (fileType) {
                var result = {};
                result.thumbnail = self.baseUrl + "/api/gateway/" + self.selectedAssetId + "/thumbnail";
                result.alternative = "M image";
                result.downloadText = downloadText;
                result.file_type = fileType;
                if (fileType == "Other") {
                    returnVal = returnVal + "&download=true"; // It's not an image, forcing the download
                    result.source = returnVal;
                    self.returnResult(result);
                }
                else {
                    Api.get(self.baseUrl + "/api/entities/" + self.selectedAssetId + "?members=MainFile").done(function (asset) {
                        var fileProperties = asset && asset.properties && asset.properties.MainFile && asset.properties.MainFile.properties;
                        if (!fileProperties) {
                            throw "Couldn't find file properties on the selected asset"; //properties => mainFle => properties => width / height
                        }
                        result.width = fileProperties.width;
                        result.height = fileProperties.height;
                        result.source = returnVal;
                        self.returnResult(result);
                    });
                }
            });
        },
        close: function () {
            this.clearSession();
            var radWindow;
            if (window.radWindow) {
                radWindow = window.radWindow;
                radWindow.Close();
            }
            else if (window.frameElement && window.frameElement.radWindow) {
                radWindow = window.frameElement.radWindow;
                radWindow.Close();
            }
            else {
                window.returnValue = null;
                window.top.returnValue = null;
                window.top.dialogClose();
            }

        },
        getSearchComponentId: function () {
            var self = this;
            if (!self.searchComponentId) {
                var postObj = {
                    page_url: self.searchPage
                };

                return Api.post(self.baseUrl + "/api/integrations/mconnector", postObj).then(function (data) {
                    self.searchComponentId = data.search_component_id;
                    if (self.searchComponentId == 0) {
                        throw "No search component found";
                    }

                    return self.searchComponentId;
                }).fail(function (xhr) { self.refreshAccessToken(); });

            }
            else {
                return $.when(null);
            }
        },
        getAssets: function () {
            var self = this;
            self.ProgressIndicator.viewModel.isVisible(true);
            self.PagerButton.viewModel.isVisible(true);
            self.NoResults.viewModel.isVisible(false);
            var datasource = self.CustomSearchDataSource;
            var searchresults = [];
            var pageSize = localStorage.getItem("pageSize");
            var currentPage = localStorage.getItem("currentPage");
            var searchTerm = self.searchTerm ? [self.searchTerm] : [];
            var itemId = pageSize * currentPage;
            var skip = pageSize * currentPage;

            var savedSelection = self.savedSelection;
            var savedsearch = self.savedSearch;
            var postObj = {
                "defaults": "SearchConfiguration",
                "culture": "en-US",
                "skip": skip,
                "take": pageSize,
                "query": "",
                "filters": [],
                "fulltext": searchTerm,
                "view": "grid",
                "component": self.searchComponentId,
                "sorting": {
                    "field": "createdon",
                    "asc": false
                }
            };

            if (!savedSelection && !savedsearch) {
                Api.post(self.baseUrl + "/api/search", postObj).done(function (data) {
                    if (!data.items || data.items.length === 0) {
                        self.ProgressIndicator.viewModel.isVisible(false);

                        if (Number(currentPage) === 0) {
                            datasource.viewModel.items([]);
                            self.NoResults.viewModel.isVisible(true);
                        } else {
                            alert("No more results.");
                        }
                        return;
                    }

                    $.each(data.items, function (idx, item) {
                        if (!item.renditions || !item.renditions.thumbnail || !item.renditions.thumbnail[0] || !item.renditions.thumbnail[0].href) return true;
                        searchresults.push({
                            itemId: itemId,
                            id: item.id,
                            title: item.properties.Title,
                            thumbnail: item.renditions.thumbnail[0].href,
                            hasPublicLink: item.HasPublicLink,
                            messages: {
                                hasPublicLinks: self.HasPublicLinkText.get("text"),
                                hasNoPublicLinks: self.HasNoPublicLinkText.get("text")
                            },
                            detailPageUrl: self.baseUrl + '/pages/en-US/AssetDetail/' + item.id,
                            stopBubble: function (event) {
                                event.stopPropagation();
                            }
                        });
                        itemId++;
                    });
                    var items = [];
                    if (Number(currentPage) > 0) {
                        items = datasource.viewModel.items();
                    }
                    datasource.viewModel.items(items.concat(searchresults));
                    self.refresh();
                    self.ProgressIndicator.viewModel.isVisible(false);

                    if ((1 + data.skip) * data.take >= data.totalItemCount) {
                        self.PagerButton.viewModel.isVisible(false);
                    }

                }).fail(function (xhr) {
                    self.refreshAccessToken();
                });
            } else if (savedSelection || savedsearch) {
                if (savedSelection) {
                    postObj.saved_selection = savedSelection;
                } else if (savedsearch && savedsearch.length > 0) {
                    postObj.query = savedsearch;
                }
                Api.post(self.baseUrl + "/api/search", postObj).done(function (data) {
                    if (!data.items || data.items.length === 0) {
                        self.ProgressIndicator.viewModel.isVisible(false);
                        if (Number(currentPage) === 0) {
                            datasource.viewModel.items([]);
                            self.NoResults.viewModel.isVisible(true);
                        } else {
                            alert("No more results.");
                        }
                        return;
                    }

                    $.each(data.items, function (idx, item) {
                        searchresults.push({
                            itemId: itemId,
                            id: item.id,
                            title: item.properties.Title,
                            thumbnail: item.renditions.thumbnail[0].href,
                            hasPublicLink: item.HasPublicLink,
                            messages: {
                                hasPublicLinks: self.HasPublicLinkText.get("text"),
                                hasNoPublicLinks: self.HasNoPublicLinkText.get("text")
                            },
                            detailPageUrl: self.baseUrl + '/pages/en-US/AssetDetail/' + item.id,
                            stopBubble: function (event) {
                                event.stopPropagation();
                            }
                        });
                        itemId++;
                    });
                    var items = [];
                    if (Number(currentPage) > 0) {
                        items = datasource.viewModel.items();
                    }
                    datasource.viewModel.items(items.concat(searchresults));
                    self.ProgressIndicator.viewModel.isVisible(false);
                    if ((1 + data.skip) * data.take >= data.totalItemCount) {
                        self.PagerButton.viewModel.isVisible(false);
                    }

                }).fail(function (xhr) { self.refreshAccessToken(); });
            };

        },
        refresh: function () {
            this.CollectionsDataSource.refresh();
            this.SavedSearchesDataSource.refresh();
        },
        clearSelectedCollection: function () {
            var self = this;
            self.savedSelection = null;
            localStorage.setItem("currentPage", 0);
            this.clearSelectedPublicLink();
            this.CollectionFilterButton.viewModel.isVisible(false);
            this.getAssets();
        },
        clearSelectedSavedSearch: function () {
            var self = this;
            self.savedSearch = null;
            localStorage.setItem("currentPage", 0);
            this.clearSelectedPublicLink();
            this.SavedSearchesFilterButton.viewModel.isVisible(false);
            this.getAssets();
        },
        clearSelectedFullText: function () {
            var self = this;
            self.searchTerm = null;
            localStorage.setItem("currentPage", 0);
            this.clearSelectedPublicLink();
            this.SearchBox.viewModel.text("");
            this.FullTextFilterButton.viewModel.isVisible(false);
            this.getAssets();
        },
        clearSelectedPublicLink: function () {
            if (this.OkButton.viewModel.isEnabled() === true) {
                localStorage.removeItem("selectedpubliclink");
                self.OkButton.viewModel.isEnabled(false);
            }
        },
        getMoreResults: function () {
            var currentPage = Number(localStorage.currentPage) + 1;
            localStorage.setItem("currentPage", currentPage);
            this.getAssets();
        },
        getMoreSavedSearches: function () {
            var self = this;
            self.currentSavedSearchPage++;
            self.getSavedSearches();
        },
        getMoreSavedSelections: function () {
            var self = this;
            self.currentSavedSelectionPage++;
            self.getCollections();
        },
        getSearchResults: function () {
            var self = this;
            if (this.OkButton.viewModel.isEnabled() === true) {
                localStorage.removeItem("selectedpubliclink");
                this.OkButton.viewModel.isEnabled(false);
            }
            // There is no need to set up savedSelection and savedSearch to be set null
            // setting them null causes the text search to ignore the selected search and selections thus resulting
            // in wrong search results
            //self.savedSelection = null;
            //self.savedSearch = null;
            var searchTerm = this.SearchBox.attributes.text;
            if (searchTerm && searchTerm.length > 0) {
                self.searchTerm = searchTerm;
                localStorage.setItem("currentPage", 0);
                var text = self.FullTextText.get("text");
                text = text.replace("{0}", searchTerm);
                this.FullTextFilterButton.viewModel.text(text + "\"  X");
                this.FullTextFilterButton.viewModel.isVisible(true);
            } else {
                self.searchTerm = "";
                localStorage.setItem("currentPage", 0);
            }
            this.getAssets();
        },
        getCollections: function () {
            var self = this;
            self.take = 10;
            self.skip = self.take * self.currentSavedSelectionPage;
            var datasource = self.CollectionsDataSource;
            self.MoreCollections.viewModel.isVisible(true);
            var itemId = 1;
            var collections = [];
            var url = "/api/savedselections?searchIdentifier=" + self.searchComponentId + "&savedSelectionType=All&skip=" + self.skip + "&take=" + self.take + "&sort=CreatedOn&order=desc"
            Api.get(self.baseUrl + url).done(function (data) {
                $.each(data.items, function (idx, item) {
                    collections.push({ itemId: itemId, id: item.id, name: item.name });
                    itemId++;
                });

                var items = [];
                if (Number(self.currentSavedSelectionPage) > 0) {
                    items = datasource.viewModel.items();
                }
                datasource.viewModel.items(items.concat(collections));

                if ((1 + self.skip) * self.take >= data.total_saved_selections) {
                    self.MoreCollections.viewModel.isVisible(false);
                }

            }).fail(function (xhr) { self.refreshAccessToken(); });
        },
        getSavedSearches: function () {
            var self = this;
            self.take = 10;
            self.skip = self.take * self.currentSavedSearchPage;
            var datasource = this.SavedSearchesDataSource;
            self.MoreSavedSearches.viewModel.isVisible(true);
            var itemId = 1;
            var savedsearches = [];
            var url = "/api/entities/query?query=Definition.name == 'M.SavedSearch' AND CreatedBy == " + self.userProfile.user_id + " AND Bool('Favourite') AND String('SearchContextIdentifier') == '" + self.searchComponentId + "'&skip=" + self.skip + "&take=" + self.take + "&members=&renditions=&culture=en-US&sort=SavedSearchName&order=asc&sortCulture="
            Api.get(self.baseUrl + url).done(function (data) {
                $.each(data.items, function (idx, item) {
                    savedsearches.push({ itemId: itemId, id: item.id, name: item.properties.SavedSearchName, query: item.properties.Query });
                    itemId++;
                });
                var items = [];
                if (Number(self.currentSavedSearchPage) > 0) {
                    items = datasource.viewModel.items();
                }
                datasource.viewModel.items(items.concat(savedsearches));

                if ((1 + self.skip) * self.take >= data.total_items) {
                    self.MoreSavedSearches.viewModel.isVisible(false);
                }
            }).fail(function (xhr) { self.refreshAccessToken(); });
        },
        getPublicLinksForAsset(id) {
            var self = this;
            var datasource = this.PublicLinksDataSource;
            var itemId = 1;
            var publiclinks = [];
            Api.get(self.baseUrl + "/api/entities/query/Definition.Name=='M.PublicLink' AND Parent('AssetToPublicLink').Id==" + id + " AND string('Status') == 'Completed' AND !bool('isDisabled')").done(function (data) {
                $.each(data.items, function (idx, item) {
                    publiclinks.push({ itemId: itemId, id: item.id, rendition: item.properties.Resource.labels['en-US'], link: item.public_link });
                    itemId++;
                });
                datasource.viewModel.items(publiclinks);
            }).fail(function (xhr) { self.refreshAccessToken(); });
        },
        setSavedSelection: function (component, obj) {
            var self = this;
            self.ProgressIndicator.viewModel.isVisible(true);

            if (!obj || !obj.id) {
                self.ProgressIndicator.viewModel.isVisible(false);
                return;
            }

            if (this.OkButton.viewModel.isEnabled() === true) {
                localStorage.removeItem("selectedpubliclink");
                self.OkButton.viewModel.isEnabled(false);
            }
            localStorage.setItem("currentPage", 0);
            self.searchTerm = null;
            self.savedSearch = null;
            self.savedSelection = obj.id;
            self.SearchBox.viewModel.text("");

            var text = self.SavedSelectionText.get("text");
            text = text.replace("{0}", obj.attributes.name);
            self.CollectionFilterButton.viewModel.text(text + "  X");
            self.CollectionFilterButton.viewModel.isVisible(true);
            // clear the searchText and savedSearches
            self.SavedSearchesFilterButton.viewModel.text("");
            self.SavedSearchesFilterButton.viewModel.isVisible(false);
            self.FullTextFilterButton.viewModel.text("");
            self.FullTextFilterButton.viewModel.isVisible(false);
            self.getAssets();
        },
        setSavedSearch: function (component, obj) {
            var self = this;
            self.ProgressIndicator.viewModel.isVisible(true);

            if (!obj || !obj.attributes || !obj.attributes.query) {
                self.ProgressIndicator.viewModel.isVisible(false);
                return;
            }

            if (this.OkButton.viewModel.isEnabled() === true) {
                localStorage.removeItem("selectedpubliclink");
                self.OkButton.viewModel.isEnabled(false);
            }
            localStorage.setItem("currentPage", 0);
            self.searchTerm = null;
            self.savedSelection = null;
            self.savedSearch = obj.attributes.query;
            this.SearchBox.viewModel.text("");
            var text = self.SavedSearchText.get("text");
            text = text.replace("{0}", obj.attributes.name);
            this.SavedSearchesFilterButton.viewModel.text(text + "  X");
            this.SavedSearchesFilterButton.viewModel.isVisible(true);
            // clear the searchText and collectionFilter
            self.CollectionFilterButton.viewModel.text("");
            self.CollectionFilterButton.viewModel.isVisible(false);
            self.FullTextFilterButton.viewModel.text("");
            self.FullTextFilterButton.viewModel.isVisible(false);
            self.getAssets();

        },
        setPublicLink: function (component, obj) {
            if (obj !== undefined) {
                if (!obj || !obj.attributes || !obj.attributes.link) {
                    throw "No 'rendition' value for public link";
                }
                if (this.OkButton.viewModel.isEnabled() === false) {
                    this.OkButton.viewModel.isEnabled(true);
                }
                localStorage.setItem("selectedpubliclink", obj.attributes.link);
            }
        },
        clearSession: function () {
            var keys = ["currentPage", "pageSize",
                "selectedpubliclink"];

            $.each(keys, function (idx, key) {
                localStorage.removeItem(key);
            });
        },
        getParameterByName: function (name) {
            var url = window.location.href;

            name = name.replace(/[\[\]]/g, '\\$&');
            var regex = new RegExp('[#&]' + name + '(=([^&#]*)|&|#|$)'),
                results = regex.exec(url);
            if (!results) return null;
            if (!results[2]) return '';
            history.pushState({}, document.title, url.replace(results[0], "")); // we got what we need, we remove it from the URL without refreshing the page so we cannot get it twice by mistake
            return decodeURIComponent(results[2].replace(/\+/g, ' '));
        },

        getAccessToken: function () {
            var self = this;
            var access_token = self.getParameterByName('access_token');

            if (access_token) {
                localStorage.setItem("access_token", access_token);
                return true;
            }
            return false;
        },
        getUserProfile: function () {
            var self = this;
            return Api.get(self.baseUrl + "/api/userprofile").then(function (data) {
                return self.userProfile = data;
            }).fail(function (xhr) {
                self.refreshAccessToken();
            });
        },
        getAuthenticationDetails: function () {
            var self = this;
            return Api.get("/api/sitecore/MContent/GetAuthenticationDetails").then(function (data) {
                self.baseUrl = data.MInstance;
                self.searchUrl = data.SearchPage;
                var clientId = data.ClientId;

                self.searchPage = data.SearchPage;
                var token = localStorage.getItem("access_token");
                if (!token) {
                    var urlHasToken = self.getAccessToken();
                    if (!urlHasToken) {
                        var url = self.baseUrl + "/OAuth/authorize?client_id=" + clientId + "&response_type=token&redirect_uri=" + window.location.origin + "/sitecore/client/Stylelabs/MApp";
                        window.location = url;
                        return false;
                    }
                }

                return true;
            });
        },
        refreshAccessToken() {
            var self = this;
            localStorage.removeItem("access_token");
            return self.getAuthenticationDetails();
        }
    });

    return SearchResults;
});